// SAOracle8.h
//

#ifndef __SAORACLE8_H__
#define __SAORACLE8_H__

#include "SqlAutmn.h"

extern "C"
{
#include "oci.h"
}

class SQL_Connection_Oracle8 : public SQL_Connection
{
public:
	SQL_Connection_Oracle8( const char *URL_ = NULL );
	virtual ~SQL_Connection_Oracle8();
	virtual PlatformType GetPlatformType() const;
	virtual void SetErrorHandler( SQL_Connection::FN_ErrorHandler );
	virtual void HandleError( int, const char *, SQL_Cursor * );
	virtual int GetErrorText( char *, int );
	virtual SQL_Connection *CreateConnection( bool ) const;
	virtual SQL_Cursor *CreateCursor();
	virtual bool Connect( const char *URL_ = NULL );
	virtual void Disconnect();
	// only for internal use
	virtual OCIEnv *GetEnvHandle() const;
	virtual OCISvcCtx *GetContextHandle() const;
	virtual OCIError *GetErrorHandle() const;
private:
	static int		InstanceCount;
	static OCIEnv	*Env;
	// members
	OCIError		*_Error;
	OCIServer		*_Server;
	OCISvcCtx		*_Context;
	OCISession		*_Session;
	// error
	OCIError		*_LastError;
	char			*_LastErrorText;
	SQL_Connection::FN_ErrorHandler	_ErrorHandler;
};

struct _Param_o8;
struct _Row_o8;

class SQL_Cursor_Oracle8 : public SQL_Cursor
{
public:
	SQL_Cursor_Oracle8( SQL_Connection * );
	virtual ~SQL_Cursor_Oracle8();
	virtual void HandleError( int, const char * );
	virtual bool Connect( SQL_Connection * );
	virtual void Disconnect();
	virtual bool IsConnected() const;
	virtual bool Commit();
	virtual bool Rollback();
	virtual bool Reset();
	virtual bool CompileSQL( const char * );
	virtual bool ExecuteSQL( const char * );
	virtual bool ExecuteProcedure( const char * );
	virtual bool FetchNext( SQL_Row * );
	virtual bool CancelQuery();
	virtual unsigned int LongWrite( const SQL_Long &, const void *, unsigned int );
	virtual unsigned int LongRead( const SQL_Long &, void *, unsigned int );
	virtual bool LongEnd( const SQL_Long & );
	virtual bool LongComplete();
	virtual SQL_Cursor &pascal operator <<( set_id );
	virtual SQL_Cursor &pascal operator <<( int );
	virtual SQL_Cursor &pascal operator <<( unsigned int );
	virtual SQL_Cursor &pascal operator <<( double );
	virtual SQL_Cursor &pascal operator <<( const char * );
	virtual SQL_Cursor &pascal operator <<( const SQL_Row & );
	virtual SQL_Cursor &pascal operator <<( SQL_Long & );
	// internal
	virtual OCIError *GetErrorHandle() const;
	virtual void SetPreFetchCount( int );
protected:
	void PositionParam();
	_Param_o8 *ParamFind( const char *, int );
	bool ParamAdd( _Param_o8 * );
	bool ParamRegister( _Param_o8 *&, bool );
	bool ParamRegisterGeneric();
	bool ParamRegisterDynamic();
	bool ParamRegisterAll();
	bool ParamRegisterClose();
	bool ParamClear();
	bool ParamBLOB( const char *, sword * );
	bool ResultRegister();
private:
	OCIError	*_Error;
	OCIStmt		*_Statement;
	char		*_QueryText;
	bool		_Returning;
	bool		_Compiled;
	bool		_Executed;
	// params
	const char	*_ParamId;
	_Param_o8	*_FirstParam;
	bool		_NoNeedExecution;
	bool		_Binded;
	bool		_DoNotClearParams;
	// result
	int			_PreFetchedCount;
	_Row_o8		*_ResultData;
	static SQL_Field::SQL_TypeMethods _Methods;
};

#endif
