// Ex01.cpp
// Demonstrates opening of connections, cursors;
// executing SQL statements and queries;
// fetching data of different types;
//

#include <windows.h>
#include <stdio.h>

#include "SqlAutmn.h"

int main()
{
	char sText[128];
	printf("Enter connect string in form of\nDATABASE/USERNAME/PASSWORD: ");
	scanf("%s", sText);
	
	SQL_Connection *pConnection = SQL_CreateConnection(
		SQL_Connection::PT_Oracle8, sText);

	SQL_Cursor *pCursor = SQL_CreateCursor(pConnection);

	try
	{
		pConnection->Connect();
		pCursor->Connect();

		try
		{
			// create a table
			pCursor->ExecuteSQL("create table TABLE_0(I INTEGER, DT DATE)");
			pCursor->Commit();
		}
		catch (SQL_Connection *)
		{} // possibly table TABLE_0 already exists

		// bind parameters
		*pCursor << 10 << "1/1/99";
		// compile and execute a statement at once
		pCursor->ExecuteSQL("insert into TABLE_0 values (:1, :2)");
		pCursor->Commit();

		*pCursor << 100 << "9/9/99";
		// execute only
		pCursor->ExecuteSQL();
		pCursor->Commit();

		// clear parameters
		pCursor->Reset();

		// fetch data
		pCursor->ExecuteSQL("select * from TABLE_0");
		SQL_Row Row;
		while (pCursor->FetchNext(&Row))
		{
			printf("I = %4d  DT = %s\n", (int)Row[1], (const char *)Row[2]);
		}

		// drop a table
		pCursor->ExecuteSQL("drop table TABLE_0");
		pCursor->Commit();

		pCursor->Disconnect();
		pConnection->Disconnect();
	}
	catch (SQL_Connection *)
	{
		pConnection->GetErrorText(sText, sizeof(sText));
		CharToOem(sText, sText); // for non-english error description
		printf("\nERROR: %s\n", sText);
	}

	SQL_DestroyCursor(pCursor);

	SQL_DestroyConnection(pConnection);

	return 0;
}
