// Ex01.cpp
// Demonstrates opening of connections, cursors;
// executing SQL statements and queries;
// fetching data of different types;
//

#include <windows.h>
#include <stdio.h>

#include "SqlAutmn.h"

const char _Text[] = "Simple text";

int main()
{
	char sText[128];
	printf("Enter connect string in form of\nDATABASE/USERNAME/PASSWORD: ");
	scanf("%s", sText);
	
	SQL_Connection *pConnection = SQL_CreateConnection(
		SQL_Connection::PT_Oracle8, sText);

	SQL_Cursor *pCursor = SQL_CreateCursor(pConnection);

	try
	{
		// make a connection
		pConnection->Connect();
		// open a cursor
		pCursor->Connect();

		try
		{
			pCursor->ExecuteSQL("create table TABLE_1(I INTEGER, L BLOB)");
			pCursor->Commit();
		}
		catch (SQL_Connection *)
		{} // possibly table TABLE_1 exists

		SQL_Long Blob("L");
		*pCursor << 10 << Blob;

		// compile and execute statement at once
		pCursor->ExecuteSQL("insert into TABLE_1 values (:1, :2)");

		// writing BLOB data
		pCursor->LongWrite(Blob, _Text, strlen(_Text)+1);
		pCursor->LongEnd(Blob);
		// complete any BLOB operations
		pCursor->LongComplete();

		// changle only the first parameter
		*pCursor << 100;
		// statement already compiled so we need to do execution only
		pCursor->ExecuteSQL();

		pCursor->LongWrite(Blob, _Text, strlen(_Text)-4);
		pCursor->LongWrite(Blob, _Text, strlen(_Text)+1);
		pCursor->LongEnd(Blob);
		pCursor->LongComplete();

		pCursor->Commit();

		// the statement will be recompiled so we need to clear previous parameters
		pCursor->Reset();

		pCursor->ExecuteSQL("select * from TABLE_1");
		// declare row for a result set
		SQL_Row Row;
		// fetching data
		while (pCursor->FetchNext(&Row))
		{
			char Buffer[128];
			// if second column is NULL
			if (Row[2].IsNull())
				strcpy(Buffer, "NULL");
			else
			{
				// getting SQL_Long variable
				Blob = Row[2];
				// reading BLOB data
				pCursor->LongRead(Blob, Buffer, sizeof(Buffer));
				pCursor->LongEnd(Blob);
				Buffer[sizeof(Buffer)-1] = '\0';
			}
			printf("I = %4d  L = %s\n", (int)Row[1], Buffer);
		}

		// need not to call pCursor->Reset because parameters were not used

		pCursor->ExecuteSQL("drop table TABLE_1");
		pCursor->Commit();

		pCursor->Disconnect();
		pConnection->Disconnect();
	}
	catch (SQL_Connection *)
	{
		pConnection->GetErrorText(sText, sizeof(sText));
		CharToOem(sText, sText);
		printf("\nERROR: %s\n", sText);
	}

	SQL_DestroyCursor(pCursor);

	SQL_DestroyConnection(pConnection);

	return 0;
}
